@@ -4,7 +4,8 @@ from abc import ABC
 
 import pydantic
 from pydantic import validator, root_validator
-from qhub.utils import namestr_regex, check_for_duplicates
+from qhub.utils import namestr_regex
+from .version import rounded_ver_parse, __version__
 
 
 class CertificateEnum(str, enum.Enum):
@@ -53,10 +54,20 @@ class Base(pydantic.BaseModel):
 class CICD(Base):
     type: CiEnum
     branch: str
+    commit_render: typing.Optional[bool] = True
     before_script: typing.Optional[typing.List[str]]
     after_script: typing.Optional[typing.List[str]]
 
 
+# ======== Generic Helm Extensions ========
+class HelmExtension(Base):
+    name: str
+    repository: str
+    chart: str
+    version: str
+    overrides: typing.Optional[typing.Dict]
+
+
 # ============== Monitoring =============
 
 
@@ -70,6 +81,7 @@ class Monitoring(Base):
 class ClearML(Base):
     enabled: bool
     enable_forward_auth: typing.Optional[bool]
+    overrides: typing.Optional[typing.Dict]
 
 
 # ============== Prefect =============
@@ -90,12 +102,6 @@ class TerraformState(Base):
     config: typing.Optional[typing.Dict[str, str]]
 
 
-class TerraformModules(Base):
-    # No longer used, so ignored, but could still be in qhub-config.yaml
-    repository: str
-    rev: str
-
-
 # ============ Certificate =============
 
 
@@ -116,14 +122,12 @@ class DefaultImages(Base):
     jupyterlab: str
     dask_worker: str
     dask_gateway: str
-    conda_store: str
 
 
 # =========== Authentication ==============
 
 
 class GitHubConfig(Base):
-    oauth_callback_url: str
     client_id: str
     client_secret: str
 
@@ -131,8 +135,6 @@ class GitHubConfig(Base):
 class Auth0Config(Base):
     client_id: str
     client_secret: str
-    oauth_callback_url: str
-    scope: typing.List[str]
     auth0_subdomain: str
 
 
@@ -184,24 +186,13 @@ class GitHubAuthentication(Authentication):
     config: GitHubConfig
 
 
-class CustomAuthentication(Authentication):
-    _typ = AuthenticationEnum.custom
-    authentication_class: str
-    config: typing.Dict[str, typing.Any]
+# ================= Keycloak ==================
 
 
-# =========== Users and Groups =============
-
-
-class User(Base):
-    uid: str
-    password: typing.Optional[str]
-    primary_group: str
-    secondary_groups: typing.Optional[typing.List[str]]
-
-
-class Group(Base):
-    gid: int
+class Keycloak(Base):
+    initial_root_password: typing.Optional[str]
+    overrides: typing.Optional[typing.Dict]
+    realm_display_name: typing.Optional[str]
 
 
 # ============== Security ================
@@ -209,13 +200,8 @@ class Group(Base):
 
 class Security(Base):
     authentication: Authentication
-    users: typing.Dict[str, User]
-    groups: typing.Dict[str, Group]
-
-    @validator("users", pre=True)
-    def validate_uderids(cls, v):
-        # raise TypeError if duplicated
-        return check_for_duplicates(v)
+    shared_users_group: typing.Optional[bool]
+    keycloak: typing.Optional[Keycloak]
 
 
 # ================ Providers ===============
@@ -258,7 +244,6 @@ class GoogleCloudPlatformProvider(Base):
 
 
 class AzureProvider(Base):
-    project: str
     region: str
     kubernetes_version: str
     node_groups: typing.Dict[str, NodeGroup]
@@ -281,7 +266,14 @@ class LocalProvider(Base):
 
 
 class Theme(Base):
-    jupyterhub: typing.Dict[str, str]
+    jupyterhub: typing.Dict[str, typing.Union[str, list]]
+
+
+# ================= Theme ==================
+
+
+class JupyterHub(Base):
+    overrides: typing.Optional[typing.Dict]
 
 
 # ================== Profiles ==================
@@ -292,7 +284,7 @@ class KubeSpawner(Base):
     cpu_guarantee: int
     mem_limit: str
     mem_guarantee: str
-    image: str
+    image: typing.Optional[str]
 
     class Config:
         extra = "allow"
@@ -312,7 +304,7 @@ class DaskWorkerProfile(Base):
     worker_cores: int
     worker_memory_limit: str
     worker_memory: str
-    image: str
+    image: typing.Optional[str]
 
     class Config:
         extra = "allow"
@@ -351,6 +343,27 @@ class CDSDashboards(Base):
     cds_hide_user_dashboard_servers: typing.Optional[bool]
 
 
+# =============== Extensions = = ==============
+
+
+class QHubExtensionEnv(Base):
+    name: str
+    value: str
+
+
+class QHubExtension(Base):
+    name: str
+    image: str
+    urlslug: str
+    private: bool = False
+    oauth2client: bool = False
+    keycloakadmin: bool = False
+    jwt: bool = False
+    qhubconfigyaml: bool = False
+    logout: typing.Optional[str]
+    envs: typing.Optional[typing.List[QHubExtensionEnv]]
+
+
 # ======== External Container Registry ========
 
 # This allows the user to set a private AWS ECR as a replacement for
@@ -397,13 +410,12 @@ class Main(Base):
     project_name: letter_dash_underscore_pydantic
     namespace: typing.Optional[letter_dash_underscore_pydantic]
     provider: ProviderEnum
+    qhub_version: str = ""
     ci_cd: typing.Optional[CICD]
     domain: str
     terraform_state: typing.Optional[TerraformState]
-    terraform_modules: typing.Optional[
-        TerraformModules
-    ]  # No longer used, so ignored, but could still be in qhub-config.yaml
     certificate: Certificate
+    helm_extensions: typing.Optional[typing.List[HelmExtension]]
     prefect: typing.Optional[Prefect]
     cdsdashboards: CDSDashboards
     security: Security
@@ -420,7 +432,41 @@ class Main(Base):
     environments: typing.Dict[str, CondaEnvironment]
     monitoring: typing.Optional[Monitoring]
     clearml: typing.Optional[ClearML]
+    tf_extensions: typing.Optional[typing.List[QHubExtension]]
+    jupyterhub: typing.Optional[JupyterHub]
+    prevent_deploy: bool = (
+        False  # Optional, but will be given default value if not present
+    )
+
+    # If the qhub_version in the schema is old
+    # we must tell the user to first run qhub upgrade
+    @validator("qhub_version", pre=True, always=True)
+    def check_default(cls, v):
+        """
+        Always called even if qhub_version is not supplied at all (so defaults to ''). That way we can give a more helpful error message.
+        """
+        if not cls.is_version_accepted(v):
+            if v == "":
+                v = "not supplied"
+            raise ValueError(
+                f"qhub_version in the config file must be equivalent to {__version__} to be processed by this version of qhub (your config file version is {v})."
+                " Install a different version of qhub or run qhub upgrade to ensure your config file is compatible."
+            )
+        return v
+
+    @classmethod
+    def is_version_accepted(cls, v):
+        return v != "" and rounded_ver_parse(v) == rounded_ver_parse(__version__)
 
 
 def verify(config):
-    Main(**config)
+    return Main(**config)
+
+
+def is_version_accepted(v):
+    """
+    Given a version string, return boolean indicating whether
+    qhub_version in the qhub-config.yaml would be acceptable
+    for deployment with the current QHub package.
+    """
+    return Main.is_version_accepted(v)
